
TitanArtifactPower = {}
local TAP = TitanArtifactPower

TAP.id = "ArtifactPower";
TAP.addon = "TitanArtifactPower";
TAP.button_label = "AP: "
TAP.menu_text = TAP.id
TAP.tooltip_header = "Artifact Information"

TAP.version = tostring(GetAddOnMetadata(TAP.addon, "Version")) or "Unknown" 
TAP.author = GetAddOnMetadata(TAP.addon, "Author") or "Unknown"
local updateTable = {TAP.id, TITAN_PANEL_UPDATE_BUTTON};
-- ******************************** Variables *******************************
local currentArtifactPower=0;
local nextRankArtifactPower=0;
local diffXP = 0;
local availableRanks=0;
local currentRank=0;
local currentIcon;
-- ******************************** Functions *******************************

function TAP.Button_OnLoad(self)

	self.registry = {
		id = TAP.id,
		version = TAP.version,
		category = "Information",	
		menuText = TAP.menu_text,
		buttonTextFunction = "TitanArtifactPower_GetButtonText", 
		tooltipTitle = TAP.tooltip_header,
		tooltipTextFunction = "TitanArtifactPower_GetTooltipText", 
		icon = "Interface\\Icons\\Icon_UpgradeStone_legendary",	
		iconWidth = 16,
		savedVariables = {
			ShowIcon = 1,
			ShowLabelText = 1,
			ShowColoredText = 1,               
		}
	};     

	self:RegisterEvent("PLAYER_ENTERING_WORLD");
	self:RegisterEvent("ARTIFACT_XP_UPDATE")
	self:RegisterEvent("ARTIFACT_CLOSE")
	self:RegisterEvent("ARTIFACT_RESPEC_PROMPT")
	self:RegisterEvent("UNIT_INVENTORY_CHANGED")
	self:RegisterEvent("ACTIVE_TALENT_GROUP_CHANGED")
	self:RegisterEvent("PLAYER_EQUIPMENT_CHANGED")
	self:RegisterEvent("UNIT_INVENTORY_CHANGED")
	
	TAP.UpdateInfo()
end

function TAP.Button_OnEvent(self, event, ...)
	TAP.UpdateInfo();
end

function TAP.Button_OnClick(self, button)
	if (button == "LeftButton") then
		if IsShiftKeyDown() then
			if(HasArtifactEquipped()) then
				local artifactLink = GetInventoryItemLink("player", 16)
				if ( availableRanks > 0 ) then
					DEFAULT_CHAT_FRAME.editBox:SetText(string.format("%s AP: %d/%d (Rank %d) [+%d]", artifactLink, currentArtifactPower, nextRankArtifactPower, currentRank, availableRanks))
				else 
					DEFAULT_CHAT_FRAME.editBox:SetText(string.format("%s AP: %d/%d (Rank %d)", artifactLink, currentArtifactPower, nextRankArtifactPower, currentRank))
				end
			end
		else
			ArtifactFrame_LoadUI()
			if ( ArtifactFrame:IsVisible() ) then
				HideUIPanel(ArtifactFrame)
			else
				SocketInventoryItem(16);
			end
		end
	end
end

function TitanArtifactPower_GetButtonText(id)
	local button, id = TitanUtils_GetButton(id, true);
	local curAPText=currentArtifactPower;
	local nextAPText=nextRankArtifactPower;
	
	button.registry.icon=currentIcon;
	
	if(currentArtifactPower>1000000) then
		curAPText=string.format("%.1fM",currentArtifactPower/1000000);
	end
	
	if(currentArtifactPower>10000) then
		curAPText=string.format("%.1fk",currentArtifactPower/1000);
	end
	
	if(nextRankArtifactPower>1000000) then
		nextAPText=string.format("%.1fM",nextRankArtifactPower/1000000);
	end
	
	if(nextRankArtifactPower>10000) then
		nextAPText=string.format("%.1fk",nextRankArtifactPower/1000);
	end
	
	local val = string.format("AP: %s/%s", curAPText , nextAPText);
	
	if(availableRanks>0) then
		val = val.." [|cff00ff00+"..availableRanks.."|r]";
	end
	
	return val;
end

function TitanArtifactPower_GetTooltipText()
	local tip = string.format("Artifact Rank: %s", currentRank);
	if(availableRanks>0) then
		tip = tip .. "[|cff00ff00+" .. availableRanks .. "|r]"
	end
	
	tip = tip .. "\n";
	tip = tip .. string.format("XP for next rank: %s", diffXP);
	
	return tip;
end

function TitanPanelRightClickMenu_PrepareArtifactPowerMenu()

end

--========= Artifact methods ==========---
function TAP.UpdateInfo()
	if ( not HasArtifactEquipped() ) then
		nextRankArtifactPower=0;
		currentArtifactPower=0;
		currentIcon=nil;
		TitanPanelPluginHandle_OnUpdate(updateTable)
		return;
	end
	
	local itemID, altItemID, name, icon, totalXP, pointsSpent, _, _, _, _, _, _ = C_ArtifactUI.GetEquippedArtifactInfo();
	
	currentArtifactPower=totalXP;
	currentIcon=icon;
	currentRank=pointsSpent;

	availableRanks = TAP.GetAvailableRanks(pointsSpent,totalXP);
	local nextRankNeededXP = TAP.GetNextRankNeededXP(pointsSpent);
	local totalRanks = pointsSpent+availableRanks;
	
	diffXP = C_ArtifactUI.GetCostForPointAtRank(totalRanks)-currentArtifactPower;

	nextRankArtifactPower=nextRankNeededXP;
	TitanPanelPluginHandle_OnUpdate(updateTable)
end

function TAP.GetAvailableRanks(pointsSpent,totalXP)
	local availableRanks = 0;
	local nextRankNeededXP = C_ArtifactUI.GetCostForPointAtRank(pointsSpent);
	while totalXP >= nextRankNeededXP and nextRankNeededXP > 0 do
		totalXP = totalXP - nextRankNeededXP;
	
		pointsSpent = pointsSpent + 1;
		availableRanks = availableRanks + 1;
	
		nextRankNeededXP = C_ArtifactUI.GetCostForPointAtRank(pointsSpent);
	end
  return availableRanks;
end

function TAP.GetNextRankNeededXP(pointsSpent)
	return C_ArtifactUI.GetCostForPointAtRank(pointsSpent);
end